#!/usr/bin/python

class Graph(dict):
	'''A graph class that uses a dictionary (sparse matrix) for edges. 
	Supports both directed and undirected graphs.
	Includes Prin-Jernik-Dijkstra and Floyd algorithms.'''
	def __init__(self,directed=True):
		dict.__init__(self)
		self.directed=True
		
	def __setitem__(self, key, value):
		'''Set weight of an edge or arc; note that edges (n1,n2) and (n2,n1) are the same'''
		try :
			v1, v2 = key
			if not self.directed :
				key=tuple([v1,v2].sort())
		except Exception, e :
			raise KeyError, e
		dict.__setitem__(self,key,value)
	
	def edges(self):
		'''List of arcs or edges'''
		return self.keys()
		
	def nodes(self):
		'''List of nodes, dynamically build from edges'''
		return reduce(lambda x,y : set(x)|set(y), self.edges(), set([]))
	
	def children(self,node):
		'''For directed graphs, lists of v_i in V such that (node, v_i) in E.
		For undirected graphs, set of all nodes connected to node by a path of length 1.
		'''
		edges = self.edges()
		child = []
		for n in self.nodes():
			if (node,n) in edges :
				child.append(n)
			if not self.directed and (n, node) in edges :
				child.append(n)
		return set(child)

	def parents(self,node):
		'''For directed graphs, lists of v_i in V such that (v_i, node) in E.
		For undirected graphs, same as children(node).
		'''
		if not self.directed :
			return self.children(node)
		edges = self.edges()
		parent = []
		for n in self.nodes():
			if (n,node) in edges :
				parent.append(n)
		return set(parent)
		
	def degree(self,n,outdegree=True):
		'''For directed graphs, number of children or parents, depending on whether outdegree or indegree is requested.
		For undirected graphs, number of edges having node as one of their vertices.
		'''
		if self.directed and not outdegree: 
			return len(self.parents(n))
		return len(self.children(n))
					
	def __getitem__(self, key):
		'''Can be invoked as:
		graph[n1,n2] : returns weight of a given edge or arc
		graph[n1] : returns children of a given node
		'''
		if key in self.nodes() :
			return self.children(key)
		try :
			v1, v2 = key
			if not self.directed :
				key=tuple([v1,v2].sort())
		except Exception, e :
			raise KeyError, e
		return dict.__getitem__(self,key)
		
	def schedule(self):
		'''Return a list of all nodes, ordered in a way that respects all precedences as given by the directed graph; raises an exception if used on an undirected graph'''
		if not self.directed : 
			raise Exception, "Undirected graphs cannot be used for scheduling problems!"
		sched = []
		ready = set([ n for n in self.nodes() if not self.degree(n,outdegree=False) ])
		while len(sched)<len(self.nodes()) :
			sched.append(ready.pop())
			ready |= set([ n for n in self.children(sched[-1]) if set(self.parents(n)).issubset(set(sched)) ])
		return sched

	def depthvisit(self, node):
		'''Return nodes in a depth first visit order for a connected component rooted in node'''
		self.visited = dict(zip(self.nodes(),[False]*len(self.nodes())))
		return self.__depthvisit(node)

	def __depthvisit(self, node):
		'''Return nodes in depth first order (main algorithm)'''
		if self.visited[node] :
			return []
		self.visited[node]=True
		return reduce(lambda x,y : x+y, [ self.__depthvisit(n) for n in self.children(node) ], [node])

	def __breadthvisit(self, node, visited):
		'''Return nodes in breadth first order (main algorithm)'''
		todo=[]
		for n in self.children(node):
			if n not in visited :
				visited.append(n)
				todo.append(n)
		for n in todo:
			visited = self.__breadthvisit(n,visited)
		return visited

	def breadthvisit(self, node):
		'''Return nodes in breadth first order, for a connected component rooted in node'''
		return self.__breadthvisit(node, [node])

	def Dijkstra(self, source):
		'''Returns minimum cost spanning tree using Prin-Jernik-Dijkstra method; 
		stopping the algorithm after reaching a specific node in Q yields Dijkstra\'s minimum path algorithm for that node.'''
		from decimal import Decimal								# The decimal module provides a representation of infinity 
		inf = Decimal('Infinity')
		dist = dict([ (v, inf) for v in self.nodes() ]) 		# Initially, all distances are set to infinity
	 	dist[source] = 0										#  except source to itself, which is zero		
	 	path = dict([ (v, None) for v in self.nodes() ])		# Here are listed the predecessors of each node in the minimum spanning tree
	 	Q = set(self.nodes())									# Q: set of vertices to visit
		while len(Q) :											# Optimize each vertex
			d, u = min([ (dist[v],v) for v in Q])				# Remove best vertex from priority queue; returns source on first iteration
			for v in self.children(u) :							# For each vertex v,
				alt = dist[u] + self[u, v]						#  check whether there is a path from source to u and then to v
				if alt < dist[v] :								#  that is shorter than the current best
					dist[v] = alt								# If so, update the distance
					path[v] = u									#  and path lists.
			Q=Q-set([u])										# Remove u from Q
		return path

	def Floyd(self):
		'''Compute distances between all node pairs using Floyd\'s Algorithm'''
		from decimal import Decimal								# The decimal module provides a representation of infinity 
		inf = Decimal('Infinity')
		nodes = self.nodes()									# Make static node list for speed
		path = {}												# Result matrix
		for i in nodes :										# For all pairs of nodes, initialize path at arc weight
			for j in nodes :									# if the arc exists, infinity otherwise
				try :
					path[i,j] = self[i,j]
				except KeyError :
					path[i,j] = inf
		for k in nodes :										# Floyd's algorithm: at each k, compute paths between all nodes that only use
			for i in nodes :									# k or all the previously considered nodes, thus incrementally exploring all
				for j in nodes :								# the graph for shortest paths
					path[i,j] = min ( path[i,j], path[i,k]+path[k,j] )
		return path

	def Kruskal(self):
		'''Return minimum cost spanning tree using Kruskal\'s algorithm'''
		result = []
		queue = [(self[e], e) for e in self.edges()]			# Build a priority queue for edges, 
		queue.sort()											#  in increasing order of weight
		partition = [ [n] for n in self.nodes() ]				# Create a forest of trees, initially without edges
		while len(queue) and len(partition) > 1:				# While we still have edges, and more than one partition
			d, e = queue.pop(0)									# Get smallest weight edge
			n0, n1 = e
			p0, p1 = None, None
			for p in partition :
				if n0 in p :
					p0 = p
				if n1 in p :
					p1 = p
			if p0!=p1 :											# If this edge allows merging two partitions, 
				partition.remove(p1)							#  use it, merge the partition and add it to the result
				partition.remove(p0)
				partition.append(p0+p1)
				result+=[e]
		return result
								
if __name__ == '__main__' :
	g=Graph()
	g[1,2]=1
	g[1,3]=4
	g[2,4]=3
	g[3,2]=1
	print g
	print [ g.degree(n) for n in g.nodes() ]
	print g.schedule()
	print g.depthvisit(1)
	print g.breadthvisit(1)
	print g.Dijkstra(1)
	print g.Floyd()
	print g.Kruskal()
	raw_input("End")
